require 'yaml'
require 'mycrm/core_ext'

require 'mycrm/attributes'
require 'mycrm/connectable'
require 'mycrm/domains'
require 'mycrm/facades'
require 'mycrm/models'
require 'mycrm/version'

module Mycrm
  class << self
    attr_reader :configuration

    private

    attr_writer :configuration

    def set_domains
      configuration.domains = YAML.load_file(domains_file)
        .depp_merge(configuration.domains.to_h.stringify_keys_deep!)
        .stringify_keys_deep!
    end

    def domains_file
      File.join(File.dirname(__FILE__), 'mycrm/resources', 'domains.yml')
    end
  end

  def self.configure
    self.configuration = Configuration.new
    yield(configuration)
    set_domains
  end

  def self.log(level, message)
    return unless configuration.logger
    configuration.logger.send level, message
  end

  class Configuration
    attr_accessor :base_uri, :username, :password, :logger, :domains, :storage, :token_storage_expiry, :api_key
  end

  class ApiError < StandardError; end
  class ResponseError < StandardError; end
  class ConnectionError < StandardError; end
end
