module Mycrm
  # facades
  module Domains
    module Rescuer
      def [](label)
        name = label.downcase.gsub(/\W/, '_')
        raise NameError, "#{label} is not a valid domain" unless constant?(name)
        create_constant(name)
      end
      alias find []

      private

      def method_missing(name, *args, &block)
        return super unless constant?(name)
        create_constant(name)
      end

      def respond_to_missing?(name, include_private = false)
        constant?(name) || super
      end

      def const_missing(name)
        return super unless constant?(name)
        create_constant(name)
      end

      def create_constant(name)
        configured_domain[name.to_underscore].tap do |val|
          self.const_set(name.to_camelized, val) unless self.const_defined?(name.to_camelized)
        end
      end

      def constant?(name)
        configured_domain && !configured_domain[name.to_underscore].nil?
      end

      def configured_domain
        Mycrm.configuration.domains[self.name.split('::').last.to_underscore]
      end
    end

    # Uncomment this line to create categories as constants on load.
    #
    # Mycrm.configuration.domains.each { |key, items| create_constant(key) }

    def self.const_missing(name)
      return super unless Mycrm.configuration.domains.key?(name.to_underscore)
      create_constant(name)
    end

    def self.create_constant(name)
      domain = Mycrm::Domains.const_set(name.to_camelized, Module.new)
      domain.extend Mycrm::Domains::Rescuer
    end
  end
end
