require_relative '../model'
require_relative '../attributes/name'
require_relative '../attributes/date'
require_relative '../attributes/address'
require_relative '../attributes/organization'

module Mycrm
  module Models
    class Adviser
      include Model

      endpoint 'lm-advisers'

      attribute :id, Integer
      attribute :active, Boolean, default: true
      attribute :source_id, Integer
      attribute :family_id, Integer
      attribute :country_id, Integer

      attribute :name, Mycrm::Attributes::Name, embedded: true
      attribute :trading_display_name, String
      attribute :dob, Mycrm::Attributes::Date, embedded: true
      attribute :address, Hash
      attribute :business_address, Hash
      attribute :email, String
      attribute :mobile, String

      attribute :business_phone, String
      attribute :abn, String
      attribute :acn, String
      attribute :individual_ASICID, String, as: :asic_id

      attribute :summary, String

      attribute :reporting_country_id, Integer
      attribute :client_type_ids, String #Array[Integer]
      attribute :role, String #Array[Integer]
      attribute :contractor_entity_id, Integer

      attribute :organization, Mycrm::Attributes::Organization, embedded: true

      attribute :uuid, String
      attribute :subscription_list, Array[Integer], coerce: false, default: nil # coerce: false set here to avoid bug https://github.com/solnic/virtus/issues/334#issuecomment-190266441

      def add_client_type(id)
        self.client_type_ids = self.client_type_ids.to_s.split(',').push(id).join(',')
      end

      def reporting_country=(country)
        self.reporting_country_id = Mycrm::Domains::Countries.find(country) rescue nil
      end

      def type=(country)
        add_client_type Mycrm::Domains::AdviserTypes.find(country) rescue nil
      end

      def source=(source)
        self.source_id = Mycrm::Domains::AdviserSources.find(source) rescue nil
      end
    end
  end
end
